{% extends "base.html" %}

{% block extrahead %}
{{ super() }}
<script>
  // Redirect starlette.io to starlette.dev
  if (window.location.hostname === 'www.starlette.io' || window.location.hostname === 'starlette.io') {
    const newUrl = window.location.href.replace(/^https?:\/\/(www\.)?starlette\.io/, 'https://starlette.dev');
    window.location.replace(newUrl);
  }
</script>
{% endblock %}

{% block announce %}
<div class="announce-wrapper">
  <div id="announce-msg">
    <a class="announce-link announce-item" href="/sponsorship/">
      ❤️ Support Starlette via <strong>sponsors</strong>!
    </a>
    <a class="announce-link announce-item" href="https://github.com/Kludex/starlette/discussions/3099" target="_blank">
      📚 Do you like the new docs? <strong>Let us know!</strong>
    </a>
  </div>
</div>
{% endblock %}
